Imports Microsoft.VisualBasic
Imports System
Imports System.Data
Imports System.Data.SqlClient

Namespace ProductList

    Public Class Products

#Region "Variables"
        Private _ProductID As String
        Private _Name As String
        Private _Description As String
        Private _Price As Decimal
        Private _CategoryID As Int32
#End Region

#Region "Constructors"
        Public Sub New()
            Me.ProductID = String.Empty
            Me.Name = String.Empty
            Me.Description = String.Empty
            Me.Price = 0
            Me.CategoryID = 0
        End Sub
        Public Sub New(ByVal pProductID As String, ByVal pName As String, ByVal pDescription As String, _
        ByVal pPrice As Decimal, ByVal pCategoryID As Int32)
            Me.ProductID = pProductID
            Me.Name = pName
            Me.Description = pDescription
            Me.Price = pPrice
            Me.CategoryID = pCategoryID
        End Sub
        Protected Sub New(ByVal reader As SqlDataReader)
            Me.ProductID = reader("ProductID").ToString()
            Me.Name = reader("Name").ToString()
            Me.Description = reader("Description").ToString()
            Me.Price = Convert.ToDecimal(reader("Price"))
            Me.CategoryID = Convert.ToInt32(reader("CategoryID"))
        End Sub
#End Region

#Region "Properties"

        Public Property ProductID() As String
            Get
                Return _ProductID
            End Get
            Set(ByVal value As String)
                _ProductID = value
            End Set
        End Property

        Public Property Name() As String
            Get
                Return _Name
            End Get
            Set(ByVal value As String)
                _Name = value
            End Set
        End Property

        Public Property Description() As String
            Get
                Return _Description
            End Get
            Set(ByVal value As String)
                _Description = value
            End Set
        End Property

        Public Property Price() As Decimal
            Get
                Return _Price
            End Get
            Set(ByVal value As Decimal)
                _Price = value
            End Set
        End Property

        Public Property CategoryID() As Int32
            Get
                Return _CategoryID
            End Get
            Set(ByVal value As Int32)
                _CategoryID = value
            End Set
        End Property

#End Region

#Region "Methods"
        ''' <summary>
        ''' Insert one product row with ProductID, Name, Description, Price and CategoryID based on the values for the 
        ''' variables ProductID, Name, Description, Price and CategoryID. You need to set values for these properties before calling
        ''' this method.
        ''' </summary>
        Public Overloads Sub InsertOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "INSERT INTO Products (ProductID, Name, Description, Price, CategoryID) VALUES" & _
                "(@ProductID, @Name, @Description, @Price, @CategoryID)"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(Sql, cn)
                    cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 50).Value = Me.ProductID
                    cmd.Parameters.Add("@Name", SqlDbType.VarChar, 200).Value = Me.Name
                    cmd.Parameters.Add("@Description", SqlDbType.Text).Value = Me.Description
                    cmd.Parameters.Add("@Price", SqlDbType.Decimal).Value = Me.Price
                    cmd.Parameters.Add("@CategoryID", SqlDbType.Int).Value = Me.CategoryID

                    cn.Open()
                    ' Execute the INSERT statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Insert one product row by using the parameter values supplied when this method is called.
        ''' </summary>
        Public Overloads Sub InsertOneRow(ByVal pProductID As String, ByVal pName As String, ByVal pDescription As String, _
        ByVal pPrice As Decimal, ByVal pCategoryID As Int32)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "INSERT INTO Products (ProductID, Name, Description, Price, CategoryID) VALUES" & _
                "(@ProductID, @Name, @Description, @Price, @CategoryID)"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 50).Value = pProductID
                    cmd.Parameters.Add("@Name", SqlDbType.VarChar, 200).Value = pName
                    cmd.Parameters.Add("@Description", SqlDbType.Text).Value = pDescription
                    cmd.Parameters.Add("@Price", SqlDbType.Decimal).Value = pPrice
                    cmd.Parameters.Add("@CategoryID", SqlDbType.Int).Value = pCategoryID

                    cn.Open()
                    ' Execute the INSERT statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Update one product row where the ProductID equals the ProductID variable in this class, you need to set values
        ''' for ProductID, Name, Description, Price and CategoryID with a constructor or with properties before using this method.
        ''' </summary>
        Public Overloads Sub UpdateOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "UPDATE Products SET Name = @Name, Description = @Description, Price = @Price, CategoryID = @CategoryID WHERE ProductID = @ProductID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 50).Value = Me.ProductID
                    cmd.Parameters.Add("@Name", SqlDbType.VarChar, 200).Value = Me.Name
                    cmd.Parameters.Add("@Description", SqlDbType.Text).Value = Me.Description
                    cmd.Parameters.Add("@Price", SqlDbType.Decimal).Value = Me.Price
                    cmd.Parameters.Add("@CategoryID", SqlDbType.Int).Value = Me.CategoryID

                    cn.Open()
                    ' Excecute the UPDATE statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Update one product row where ProductID equals the supplied parameter pProductID , you need to add 
        ''' five parameters to use this method.
        ''' </summary>
        Public Overloads Sub UpdateOneRow(ByVal pProductID As String, ByVal pName As String, ByVal pDescription As String, _
        ByVal pPrice As Decimal, ByVal pCategoryID As Int32)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "UPDATE Products SET Name = @Name, Description = @Description, Price = @Price, CategoryID = @CategoryID WHERE ProductID = @ProductID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 50).Value = pProductID
                    cmd.Parameters.Add("@Name", SqlDbType.VarChar, 200).Value = pName
                    cmd.Parameters.Add("@Description", SqlDbType.Text).Value = pDescription
                    cmd.Parameters.Add("@Price", SqlDbType.Decimal).Value = pPrice
                    cmd.Parameters.Add("@CategoryID", SqlDbType.Int).Value = pCategoryID

                    cn.Open()
                    ' Excecute the UPDATE statement
                    cmd.ExecuteNonQuery()
                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Select one product row where ProductID equals the value for the variable ProductID in this class, which you
        ''' have to set with a constructor or a property before using this method. The selected row is used to set the
        ''' properties for ProductID, Name, Description, Price and CategoryID so you have to get values from the properties to get the
        ''' selected data.
        ''' </summary>
        Public Overloads Sub SelectOneRow()

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Products WHERE ProductID = @ProductID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 50).Value = Me.ProductID

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            Me.ProductID = reader("ProductID").ToString()
                            Me.Name = reader("Name").ToString()
                            Me.Description = reader("Description").ToString()
                            Me.Price = Convert.ToDecimal(reader("Price"))
                            Me.CategoryID = Convert.ToInt32(reader("CategoryID"))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub

        ''' <summary>
        ''' Select one product row where ProductID equals pProductID, you need supply a parameter value for ProductID to use
        ''' this method. The selected row is used to set values for the properties: ProductID, Name, Description, Price and 
        ''' CategoryID. You need to get the values from these properties after calling this method.
        ''' </summary>
        Public Overloads Sub SelectOneRow(ByVal pProductID As Int32)

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Products WHERE ProductID = @ProductID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@ProductID", SqlDbType.VarChar, 50).Value = pProductID

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader(CommandBehavior.SingleRow)
                        While reader.Read()
                            Me.ProductID = reader("ProductID").ToString()
                            Me.Name = reader("Name").ToString()
                            Me.Description = reader("Description").ToString()
                            Me.Price = Convert.ToDecimal(reader("Price"))
                            Me.CategoryID = Convert.ToInt32(reader("CategoryID"))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try
        End Sub
        ''' <summary>
        ''' Select all product rows to an ArrayList of product objects (ProductID, Name, Description, Price and CategoryID). An 
        ''' ArrayList of objects can be used directly as a datasource for a repeater control or other listcontrols.
        ''' </summary>
        Public Function SelectAllToArrayList() As ArrayList

            ' Declare AllToArrayList as a new ArrayList
            Dim AllToArrayList As New ArrayList

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Products"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader()
                        While reader.Read()
                            ' Add a category object to the ArrayList
                            AllToArrayList.Add(New Products(reader))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a ArrayList
            Return AllToArrayList

        End Function

        ''' <summary>
        ''' Select all product rows to an ArrayList of product objects (ProductID, Name, Description, Price and CategoryID) 
        ''' where the CategoryID equals pCategoryID value. An ArrayList of objects can be used directly as 
        ''' a datasource for a repeater control or other datalists.
        ''' </summary>
        Public Function SelectAllToArrayList(ByVal pCategoryID As Int32) As ArrayList

            ' Declare AllToArrayList as a new ArrayList
            Dim AllToArrayList As New ArrayList

            ' The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
            ' application crash in such cases.
            Try
                Dim ConnString As String = ConfigurationManager.ConnectionStrings("ConnectionString").ToString()
                Dim sql As String = "SELECT * FROM Products WHERE CategoryID = @CategoryID"

                ' The Using block is used to call dispose automatically even if there are an exception, this means 
                ' that we dont need "finally" in the Try/Catch statement.
                Using cn As New SqlConnection(ConnString), _
                      cmd As New SqlCommand(sql, cn)
                    cmd.Parameters.Add("@CategoryID", SqlDbType.Int).Value = pCategoryID

                    cn.Open()

                    ' The Using block is used to call dispose automatically even if there are an exception, this means 
                    ' that we dont need "finally" in the Try/Catch statement. The SqlDataReader does not have a 
                    ' constructor and thats why we dont write the "New" keyword.
                    Using reader As SqlDataReader = cmd.ExecuteReader()
                        While reader.Read()
                            ' Add a category object to the ArrayList
                            AllToArrayList.Add(New Products(reader))
                        End While
                    End Using

                End Using
            Catch ex As SqlException
                Throw New Exception("ProductList", ex)
            End Try

            ' Return a ArrayList
            Return AllToArrayList

        End Function
#End Region

    End Class

End Namespace
