using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Data.SqlClient;
using System.Collections;

namespace ProductList
{
    public class Categories
    {

        #region Variables

        private Int32 _CategoryID;
        private string _LinkName;
        private string _Description;
        private Int32 _HeadCategoryID;

        #endregion

        #region Constructors

        public Categories()
        {
            this.CategoryID = 0;
            this.LinkName = string.Empty;
            this.Description = string.Empty;
            this.HeadCategoryID = 0;
        }

        public Categories(string pLinkName, string pDescription, Int32 pHeadCategoryID)
        {
            this.LinkName = pLinkName;
            this.Description = pDescription;
            this.HeadCategoryID = pHeadCategoryID;
        }

        public Categories(Int32 pCategoryID, string pLinkName, string pDescription, Int32 pHeadCategoryID)
        {
            this.CategoryID = pCategoryID;
            this.LinkName = pLinkName;
            this.Description = pDescription;
            this.HeadCategoryID = pHeadCategoryID;
        }

        protected Categories(SqlDataReader reader)
        {
            this.CategoryID = Convert.ToInt32(reader["CategoryID"]);
            this.LinkName = reader["LinkName"].ToString();
            this.Description = reader["Description"].ToString();
            this.HeadCategoryID = Convert.ToInt32(reader["HeadCategoryID"]);
        }

        #endregion

        #region Properties

        public Int32 CategoryID
        {
            get { return _CategoryID; }
            set { _CategoryID = value; }
        }

        public string LinkName
        {
            get { return _LinkName; }
            set { _LinkName = value; }
        }

        public string Description
        {
            get { return _Description; }
            set { _Description = value; }
        }

        public Int32 HeadCategoryID
        {
            get { return _HeadCategoryID; }
            set { _HeadCategoryID = value; }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Insert one category row with LinkName, Description and HeadCategoryID based on the values for the 
        /// variables LinkName, Description and HeadCategoryID. You need to set values for these properties before calling
        /// this method, CategoryId is a identity column that increments with 1.
        /// </summary>
        public void InsertOneRow()
        {

            // Declare variables for a connection string and a INSERT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "INSERT INTO Categories (LinkName, Description, HeadCategoryID) VALUES" 
            + "(@LinkName, @Description, @HeadCategoryID)";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@LinkName", this.LinkName);
                cmd.Parameters.AddWithValue("@Description", this.Description);
                cmd.Parameters.AddWithValue("@HeadCategoryID", this.HeadCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection
                    cn.Open();

                    // Execute the INSERT statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Insert one category row by using the parameter values supplied when this method is called, 
        /// the CategoryId column is a identity that increments with 1.
        /// </summary>
        public void InsertOneRow(string pLinkName, string pDescription, Int32 pHeadCategoryID)
        {
            // Declare variables for a connection string and a INSERT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "INSERT INTO Categories (LinkName, Description, HeadCategoryID) VALUES" 
            + "(@LinkName, @Description, @HeadCategoryID)";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@LinkName", pLinkName);
                cmd.Parameters.AddWithValue("@Description", pDescription);
                cmd.Parameters.AddWithValue("@HeadCategoryID", pHeadCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the INSERT statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Update one category row where the CategoryId equals the CategoryId variable in this class, you need to set values
        /// for CategoryId, LinkName, Description and HeadCategoryID with a constructor or with properties before using this method.
        /// </summary>
        public void UpdateOneRow()
        {
            // Declare a connection string and a UPDATE statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "UPDATE Categories SET LinkName = @LinkName, Description = @Description, HeadCategoryID "
            + "= @HeadCategoryID WHERE CategoryID = @Id";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@Id", this.CategoryID);
                cmd.Parameters.AddWithValue("@LinkName", this.LinkName);
                cmd.Parameters.AddWithValue("@Description", this.Description);
                cmd.Parameters.AddWithValue("@HeadCategoryID", this.HeadCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Excecute the UPDATE statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Update one category row where the CategoryId equals the supplied parameter pCategoryId, you need to add 
        /// four parameters to use this method.
        /// </summary>
        public void UpdateOneRow(Int32 pCategoryID, string pLinkName, string pDescription, Int32 pHeadCategoryID)
        {
            // Declare a connection string and a UPDATE statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "UPDATE Categories SET LinkName = @LinkName, Description = @Description, HeadCategoryID "
            + "= @HeadCategoryID WHERE CategoryID = @Id";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@Id", pCategoryID);
                cmd.Parameters.AddWithValue("@LinkName", pLinkName);
                cmd.Parameters.AddWithValue("@Description", pDescription);
                cmd.Parameters.AddWithValue("@HeadCategoryID", pHeadCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Excecute the UPDATE statement.
                    cmd.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Select one category row where the CategoryId equals the value for the variable CategoryId in this class, which you
        /// have to set with a constructor or a property before using this method. The selected row is used to set the
        /// properties for CategoryId, LinkName, Description and HeadCategoryID so you have to get values from the properties to get the
        /// selected data.
        /// </summary>
        public void SelectOneRow()
        {
            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Categories WHERE CategoryID = @CategoryID";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why 
                // we dont write the "New" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@CategoryID", this.CategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        this.CategoryID = Convert.ToInt32(reader["CategoryID"]);
                        this.LinkName = reader["LinkName"].ToString();
                        this.Description = reader["Description"].ToString();
                        this.HeadCategoryID = Convert.ToInt32(reader["HeadCategoryID"]);
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        /// <summary>
        /// Select one category row where CategoryId equals pCategoryID, you need supply a parameter value for CategoryId to use
        /// this method. The selected row is used to set values for the properties: CategoryId, LinkName, Description and
        /// HeadCategoryID. You need to get the values from these properties after calling this method.
        /// </summary>
        public void SelectOneRow(Int32 pCategoryID)
        {
            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Categories WHERE CategoryID = @CategoryID";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats 
                // why we dont write the "new" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@CategoryID", pCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        this.CategoryID = Convert.ToInt32(reader["CategoryID"]);
                        this.LinkName = reader["LinkName"].ToString();
                        this.Description = reader["Description"].ToString();
                        this.HeadCategoryID = Convert.ToInt32(reader["HeadCategoryID"]);
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        /// <summary>
        /// Select all category rows to an ArrayList of category objects (CategoryId, LinkName, Description, HeadCategoryID). An 
        /// ArrayList of objects can be used directly as a datasource for a repeater control or other listcontrols.
        /// </summary>
        public ArrayList SelectAllToArrayList()
        {
            // Declare AllToArrayList as a new ArrayList
            ArrayList AllToArrayList = new ArrayList();

            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Categories";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why we 
                // dont write the "New" keyword.
                SqlDataReader reader = null;

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader();

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Add a category object to the ArrayList
                        AllToArrayList.Add(new Categories(reader));
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a ArrayList
            return AllToArrayList;
        }

        /// <summary>
        /// Select all category rows to an ArrayList of category objects (CategoryId, LinkName, Description, HeadCategoryID) 
        /// where the HeadCategoryID equals pHeadCategoryID value. An ArrayList of objects can be used directly as 
        /// a datasource for a repeater control or other datalists.
        /// </summary>
        public ArrayList SelectAllToArrayList(Int32 pHeadCategoryID)
        {

            // Declare AllToArrayList as a new ArrayList
            ArrayList AllToArrayList = new ArrayList();

            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Categories WHERE HeadCategoryID = @HeadCategoryID";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats 
                // why we dont write the "new" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@HeadCategoryID", pHeadCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader();

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Add a category object to the ArrayList
                        AllToArrayList.Add(new Categories(reader));
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a ArrayList
            return AllToArrayList;
        }

        #endregion
    }
}

