using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Data.SqlClient;
using System.Collections;

namespace ProductList
{
    public class Products
    {
        #region Variables

        private string _ProductID;
        private string _Name;
        private string _Description;
        private decimal _Price;
        private Int32 _CategoryID;

        #endregion
        
        #region Constructors

        public Products()
        {
            this.ProductID = string.Empty;
            this.Name = string.Empty;
            this.Description = string.Empty;
            this.Price = 0m;
            this.CategoryID = 0;
        }

        public Products(string pProductID, string pName, string pDescription, decimal pPrice, Int32 pCategoryID)
        {
            this.ProductID = pProductID;
            this.Name = pName;
            this.Description = pDescription;
            this.Price = pPrice;
            this.CategoryID = pCategoryID;
        }

        protected Products(SqlDataReader reader)
        {
            this.ProductID = reader["ProductID"].ToString();
            this.Name = reader["Name"].ToString();
            this.Description = reader["Description"].ToString();
            this.Price = Convert.ToDecimal(reader["Price"]);
            this.CategoryID = Convert.ToInt32(reader["CategoryID"]);
        }

        #endregion

        #region Properties

        public string ProductID
        {
            get { return _ProductID; }
            set { _ProductID = value; }
        }

        public string Name
        {
            get { return _Name; }
            set { _Name = value; }
        }

        public string Description
        {
            get { return _Description; }
            set { _Description = value; }
        }

        public decimal Price
        {
            get { return _Price; }
            set { _Price = value; }
        }

        public Int32 CategoryID
        {
            get { return _CategoryID; }
            set { _CategoryID = value; }
        }

        #endregion

        #region Methods
        
        /// <summary>
        /// Insert one product row with ProductID, Name, Description, Price and CategoryID based on the values for the 
        /// variables ProductID, Name, Description, Price and CategoryID. You need to set values for these properties before calling
        /// this method.
        /// </summary>
        public void InsertOneRow()
        {
            // Declare variables for a connection string and a INSERT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "INSERT INTO Products (ProductID, Name, Description, Price, CategoryID) VALUES" 
            + "(@ProductID, @Name, @Description, @Price, @CategoryID)";

                // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
                // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@ProductID", this.ProductID);
                cmd.Parameters.AddWithValue("@Name", this.Name);
                cmd.Parameters.AddWithValue("@Description", this.Description);
                cmd.Parameters.AddWithValue("@Price", this.Price);
                cmd.Parameters.AddWithValue("@CategoryID", this.CategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {

                    // Open the connection.
                    cn.Open();

                    // Execute the INSERT statement.
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                }
            }
        }

        /// <summary>
        /// Insert one product row by using the parameter values supplied when this method is called.
        /// </summary>
        public void InsertOneRow(string pProductID, string pName, string pDescription, decimal pPrice, Int32 pCategoryID)
        {
            // Declare variables for a connection string and a INSERT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "INSERT INTO Products (ProductID, Name, Description, Price, CategoryID) VALUES" 
            + "(@ProductID, @Name, @Description, @Price, @CategoryID)"; 

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@ProductID", pProductID);
                cmd.Parameters.AddWithValue("@Name", pName);
                cmd.Parameters.AddWithValue("@Description", pDescription);
                cmd.Parameters.AddWithValue("@Price", pPrice);
                cmd.Parameters.AddWithValue("@CategoryID", pCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the INSERT statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                }
            }
        }

        /// <summary>
        /// Update one product row where the ProductID equals the ProductID variable in this class, you need to set values
        /// for ProductID, Name, Description, Price and CategoryID with a constructor or with properties before using this method.
        /// </summary>
        public void UpdateOneRow()
        {
            // Declare variables for a connection string and a UPDATE statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "UPDATE Products SET Name = @Name, Description = @Description, Price = @Price, CategoryID = @CategoryID WHERE ProductID = @ProductID";
 
            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@ProductID", this.ProductID);
                cmd.Parameters.AddWithValue("@Name", this.Name);
                cmd.Parameters.AddWithValue("@Description", this.Description);
                cmd.Parameters.AddWithValue("@Price", this.Price);
                cmd.Parameters.AddWithValue("@CategoryID", this.CategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Excecute the UPDATE statement
                    cmd.ExecuteNonQuery();

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Update one product row where ProductID equals the supplied parameter pProductID , you need to add 
        /// five parameters to use this method.
        /// </summary>
        public void UpdateOneRow(string pProductID, string pName, string pDescription, decimal pPrice, Int32 pCategoryID)
        {
            // Declare variables for a connection string and a UPDATE statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "UPDATE Products SET Name = @Name, Description = @Description, Price = @Price, CategoryID = @CategoryID WHERE ProductID = @ProductID";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Add parameters.
                cmd.Parameters.AddWithValue("@ProductID", pProductID);
                cmd.Parameters.AddWithValue("@Name", pName);
                cmd.Parameters.AddWithValue("@Description", pDescription);
                cmd.Parameters.AddWithValue("@Price", pPrice);
                cmd.Parameters.AddWithValue("@CategoryID", pCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection
                    cn.Open();

                    // Excecute the UPDATE statement
                    cmd.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Select one product row where ProductID equals the value for the variable ProductID in this class, which you
        /// have to set with a constructor or a property before using this method. The selected row is used to set the
        /// properties for ProductID, Name, Description, Price and CategoryID so you have to get values from the properties to get the
        /// selected data.
        /// </summary>
        public void SelectOneRow()
        {
            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Products WHERE ProductID = @ProductID";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats 
                // why we dont write the "New" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@ProductID", this.ProductID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        this.ProductID = reader["ProductID"].ToString();
                        this.Name = reader["Name"].ToString();
                        this.Description = reader["Description"].ToString();
                        this.Price = Convert.ToDecimal(reader["Price"]);
                        this.CategoryID = Convert.ToInt32(reader["CategoryID"]);
                    }

                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        /// <summary>
        /// Select one product row where ProductID equals pProductID, you need supply a parameter value for ProductID to use
        /// this method. The selected row is used to set values for the properties: ProductID, Name, Description, Price and 
        /// CategoryID. You need to get the values from these properties after calling this method.
        /// </summary>
        public void SelectOneRow(Int32 pProductID)
        {
            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Products WHERE ProductID = @ProductID";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats 
                // why we dont write the "new" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@ProductID", pProductID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader(CommandBehavior.SingleRow);

                    // Loop the reader.
                    while (reader.Read())
                    {
                        this.ProductID = reader["ProductID"].ToString();
                        this.Name = reader["Name"].ToString();
                        this.Description = reader["Description"].ToString();
                        this.Price = Convert.ToDecimal(reader["Price"]);
                        this.CategoryID = Convert.ToInt32(reader["CategoryID"]);
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        /// <summary>
        /// Select all product rows to an ArrayList of product objects (ProductID, Name, Description, Price and CategoryID). An 
        /// ArrayList of objects can be used directly as a datasource for a repeater control or other listcontrols.
        /// </summary>
        public ArrayList SelectAllToArrayList()
        {
            // Declare AllToArrayList as a new ArrayList.
            ArrayList AllToArrayList = new ArrayList();

            // Declare variables for a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Products";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats 
                // why we dont write the "new" keyword.
                SqlDataReader reader = null;

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader();

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Add a category object to the ArrayList
                        AllToArrayList.Add(new Products(reader));
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a ArrayList
            return AllToArrayList;
        }

        /// <summary>
        /// Select all product rows to an ArrayList of product objects (ProductID, Name, Description, Price and CategoryID) 
        /// where the CategoryID equals pCategoryID value. An ArrayList of objects can be used directly as 
        /// a datasource for a repeater control or other datalists.
        /// </summary>
        public ArrayList SelectAllToArrayList(Int32 pCategoryID)
        {
            // Declare AllToArrayList as a new ArrayList
            ArrayList AllToArrayList = new ArrayList();

            // Declare a connection string and a SELECT statement.
            string ConnString = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
            string sql = "SELECT * FROM Products WHERE CategoryID = @CategoryID";

            // Create a SqlConnection. The using block is used to call dispose automatically even if there are an exception, this means 
            // that we dont need "finally" in the Try/Catch statement.
            using (SqlConnection cn = new SqlConnection(ConnString))
            {
                // Create a SqlCommand. The SqlDataReader does not have a constructor and thats why we dont 
                // write the "new" keyword.
                SqlCommand cmd = new SqlCommand(sql, cn);

                // Create a SqlDataReader. The SqlDataReader does not have a constructor and thats why we dont 
                // write the "new" keyword.
                SqlDataReader reader = null;

                // Add parameters.
                cmd.Parameters.AddWithValue("@CategoryID", pCategoryID);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to avoid having our
                // application crash in such cases.
                try
                {
                    // Open the Connection.
                    cn.Open();

                    // Execute the SELECT statement and fill the reader with data.
                    reader = cmd.ExecuteReader();

                    // Loop the reader.
                    while (reader.Read())
                    {
                        // Add a category object to the ArrayList
                        AllToArrayList.Add(new Products(reader));
                    }
                }
                catch (Exception ex)
                {
                    throw new Exception("ProductList", ex);
                }
                finally
                {
                    // Dispose of objects to avoid memory leakage.
                    cmd.Dispose();

                    // Close the reader if it not is null.
                    if (reader != null)
                        reader.Close();
                }
            }

            // Return a ArrayList
            return AllToArrayList;
        }

        #endregion
    }
}

